#import "AppDelegate.h"


@implementation AppDelegate

#pragma mark ======= Initialization =======
- (void)awakeFromNib;
{
    // Check the standard user defaults to see if we have a previously loaded project file.
    NSString *projectPath = [[NSUserDefaults standardUserDefaults] stringForKey:@"ProjectPath"];
    
    if( projectPath != nil )
    {
        // If so, we make sure the file still exists.
        if( ![[NSFileManager defaultManager] fileExistsAtPath:projectPath] )
        {
            // If it doesn't exist, we prompt the user to create a new one
            [self newProjectFile:nil];
            
        }
        else
        {
            // If it does exist, we go ahead and open the project and
            // set it as our currentProjectFilepath.
            [self openProject:projectPath];
        }
    }
    else
    {
        // If we had no project file specified, we prompt the user to create
        // a new project file and allocate the files array with a capacity of 10.
        [self newProjectFile:nil];
    }
    
    // Register out table view to receive double clicks.
    [tableView setTarget:self];
    [tableView setDoubleAction:@selector(tableViewDoubleClick:)];
}


- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
    // This gets called when the user double-clicks a project file in the Finder.
    [self openProject:filename];
    [self setupProject:filename];
    return YES;
}

- (void)application:(NSApplication *)sender openFiles:(NSArray *)filenames
{
    // This gets called when the user opens multiple files from the Finder.
    
    // We only take the first file.
    if( [filenames count] > 0 )
    {
        [self application:sender openFile:[filenames objectAtIndex:0]];
    }
}


#pragma mark -

#pragma mark ======= Project File Management =======
- (IBAction)newProjectFile:(id)sender;
{
    NSSavePanel *savePanel;
    
    savePanel = [NSSavePanel savePanel];
    [savePanel setExtensionHidden:YES];
    // Only allow our specific file type
    [savePanel setRequiredFileType:@"pdfkeeper"];
    // Disable ability to show file extension
    [savePanel setCanSelectHiddenExtension:NO];
    // Make sure files are not treated as directories
    [savePanel setTreatsFilePackagesAsDirectories:NO];
    if( [savePanel runModal] == NSOKButton )
    {
        // Save the project file
        [self saveProject:[savePanel filename]];
        // Initialize our pdfFiles array that we use for the list view model
        pdfFiles = [[NSMutableArray alloc] initWithCapacity:10];
        // Set the window title to the filename path
        [mainWindow setTitle:[savePanel filename]];
    }
    
}

- (IBAction)saveProjectFile:(id)sender;
{
    [self saveProject:currentProjectFilepath];
}

- (void)saveProject:(NSString*)filepath;
{
    // Create our project file if it doesn't exist ensuring that we set the
    // file extesion to be hidden.
    NSNumber *num = [NSNumber numberWithBool:YES];
    NSDictionary *attribs = [NSDictionary dictionaryWithObjectsAndKeys:num, NSFileExtensionHidden, nil];
    if( ![[NSFileManager defaultManager] fileExistsAtPath:filepath] )
    {
        [[NSFileManager defaultManager] createDirectoryAtPath:filepath attributes:attribs];
    }

    // Set the current project file path.
    currentProjectFilepath = filepath;

    // Set up project file internal directories
    NSString *contentsPath = [currentProjectFilepath stringByAppendingPathComponent:@"Contents"];
    // If this file is new, we'll need to create the Contents directory
    if( ![[NSFileManager defaultManager] fileExistsAtPath:contentsPath] )
        [[NSFileManager defaultManager] createDirectoryAtPath:contentsPath withIntermediateDirectories:YES attributes:nil error:nil];
    NSString *resourcesPath = [contentsPath stringByAppendingPathComponent:@"Resources"];
    // If this file is new, we'll need to create the Contents/Resources directory
    if( ![[NSFileManager defaultManager] fileExistsAtPath:resourcesPath] )
        [[NSFileManager defaultManager] createDirectoryAtPath:resourcesPath withIntermediateDirectories:YES attributes:nil error:nil];

    // Set the path for our property list file.
    NSString *plistFile = [contentsPath stringByAppendingPathComponent:@"Info.plist"];
    
    // If it already exists in the project, delete it.
    if( [[NSFileManager defaultManager] fileExistsAtPath:plistFile] )
        [[NSFileManager defaultManager] removeFileAtPath:plistFile handler:nil];
    
    // Create a dictionary to use to write out our property list file.
    NSDictionary *output;
    output = [NSDictionary dictionaryWithObjectsAndKeys:pdfFiles, @"Files", nil];
    
    // Write the file.
    [output writeToFile:plistFile atomically:YES];

    // Set the current project path in our user defaults, so that the next
    // time the app loads, it will load this project.
    [[NSUserDefaults standardUserDefaults] setObject:filepath forKey:@"ProjectPath"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (IBAction)saveProjectFileAs:(id)sender;
{
    NSSavePanel *savePanel;
    
    savePanel = [NSSavePanel savePanel];
    [savePanel setExtensionHidden:YES];
    // Only allow our specific file type
    [savePanel setRequiredFileType:@"pdfkeeper"];
    // Disable ability to show file extension
    [savePanel setCanSelectHiddenExtension:NO];
    // Make sure files are not treated as directories
    [savePanel setTreatsFilePackagesAsDirectories:NO];
    if( [savePanel runModal] == NSOKButton )
    {
        // Make a copy of the entire project
        [[NSFileManager defaultManager] copyPath:currentProjectFilepath toPath:[savePanel filename] handler:nil];
        
        // Set our currentProjectFilepath to the new filename.
        currentProjectFilepath = [savePanel filename];
    }
    
}

- (IBAction)openProjectFile:(id)sender;
{
    NSOpenPanel *openPanel;
	
    openPanel = [NSOpenPanel openPanel];
    [openPanel setCanChooseDirectories:NO];
    [openPanel setAllowsMultipleSelection:NO];
    [openPanel setResolvesAliases:YES];
    [openPanel setCanChooseFiles:YES];

	// Only allow the user to select our specific file type.
    NSArray *fileTypes = [NSArray arrayWithObjects: @"pdfkeeper", nil];
    
    if ([openPanel runModalForTypes:fileTypes] == NSOKButton)
    {
        [self openProject:[openPanel filename]];
    }
    
}

- (void)openProject:(NSString*)filepath;
{
    currentProjectFilepath = filepath;
    // Set the window title to our filepath.
    [mainWindow setTitle:filepath];
    // Setup the project
    [self setupProject:filepath];
    
    // Set our project path in the standard user defaults
    [[NSUserDefaults standardUserDefaults] setObject:filepath forKey:@"ProjectPath"];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    NSBundle *contextBundle = [NSBundle bundleWithPath:filepath];
    if (![contextBundle infoDictionary]) return;
    
    NSLog(@"The info of the bundle: %@", [contextBundle infoDictionary]);

}

#pragma mark -

#pragma mark ======= PDF File management ========
- (IBAction)addPDFFile:(id)sender;
{
    NSOpenPanel *openPanel;
	
    openPanel = [NSOpenPanel openPanel];
    [openPanel setCanChooseDirectories:NO];
    [openPanel setAllowsMultipleSelection:NO];
    [openPanel setResolvesAliases:YES];
    [openPanel setCanChooseFiles:YES];
	
    NSArray *fileTypes = [NSArray arrayWithObjects: @"pdf", nil];
    
    if ([openPanel runModalForTypes:fileTypes] == NSOKButton)
    {
        // Build our destination filename based upon the current project
        // file path and the Contents/Resources path below it.
        NSString *filename = [openPanel filename];
        NSString *displayName = [[NSFileManager defaultManager] displayNameAtPath:filename];
        NSString *destFile = [currentProjectFilepath stringByAppendingPathComponent:@"Contents"];
        destFile = [destFile stringByAppendingPathComponent:@"Resources"];
        destFile = [destFile stringByAppendingPathComponent:displayName];
        
        // Use the NSFileManager to copy the file
        [[NSFileManager defaultManager] copyPath:filename toPath:destFile handler:nil];

        if( pdfFiles != nil )
        {
            // Add the filename to our list of files.
            [pdfFiles addObject:displayName];
            // Save the project.
            [self saveProject:currentProjectFilepath];
            // Reload the table view
            [tableView reloadData];
        }
    }
    
}

#pragma mark -

- (void)setupProject:(NSString*)projectPath;
{
    currentProjectFilepath = projectPath;
    
    // Load the file bundle to obtain the bundle dictionary
    NSBundle *contextBundle = [NSBundle bundleWithPath:projectPath];
    if (![contextBundle infoDictionary]) return;
    
    // The infoDictionary represents the Info.plist file. We load it
    // and grab the array of files using the key "Files"
    NSArray *files = [[contextBundle infoDictionary] objectForKey:@"Files"];
    
    // If we have a list of files, we initialize our array model with them.
    // Otherwise we initialize the array with a capacity of 10
    if( files != nil )
    {
        pdfFiles = [[NSMutableArray alloc] initWithArray:files];
    }
    else
    {
        pdfFiles = [[NSMutableArray alloc] initWithCapacity:10];
    }
    
    // Reload the table view
    [tableView reloadData];
}

#pragma mark ======= TableView Delegates =======
- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
    if( pdfFiles != nil )
        return [pdfFiles count];
    
    return 0;
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn
            row:(int)row
{
    if( pdfFiles != nil )
    {
        // If the checkbox is checked, we display the full path
        // to each PDF file. Otherwise, we just show the filename.
        if( [showFullPDFPathButton state] == NSOnState )
        {
            NSString *fullPath = [currentProjectFilepath stringByAppendingPathComponent:@"Contents"];
            fullPath = [fullPath stringByAppendingPathComponent:@"Resources"];
            fullPath = [fullPath stringByAppendingPathComponent:[pdfFiles objectAtIndex:row]];
            return fullPath;
        }
        else
        {
            return [pdfFiles objectAtIndex:row];
        }
    }
    return @"";
}

- (IBAction)tableViewDoubleClick:(id)sender;
{
    // Build our path for the PDF file that was just double clicked based
    // upon the current proejct file path and the Contents/Resources sub path
    // below it.
    NSString *fullPath = [currentProjectFilepath stringByAppendingPathComponent:@"Contents"];
    fullPath = [fullPath stringByAppendingPathComponent:@"Resources"];
    fullPath = [fullPath stringByAppendingPathComponent:[pdfFiles objectAtIndex:[sender selectedRow]]];
    
    // Load the PDF file into an NSData object.
    NSData *data = [[NSData alloc] initWithContentsOfFile:fullPath];

    // Load the PDF document from the NSData.
    PDFDocument *doc = [[PDFDocument alloc] initWithData:data];
    
    // Set the document in the PDFView
    [pdfView setDocument:doc];
    
    // Center The PDF display window
    [pdfDisplayWindow center];
    // Show the PDF display window
    [pdfDisplayWindow makeKeyAndOrderFront:sender];
}

#pragma mark -

#pragma mark ======= Checkbox Button Delegate =======
- (IBAction)buttonStateChanged:(id)sender;
{
    // When the button state changes, force the table view
    // to reaload its data.
    [tableView reloadData];
}
#pragma mark -

@end
